@@ -1,30 +1,33 @@
 """Functions for loading IBL ephys and trial data using the Open Neurophysiology Environment."""
+from dataclasses import dataclass
 import logging
 import os
+from pathlib import Path
 
 import numpy as np
 import pandas as pd
 from scipy.interpolate import interp1d
 
-from one.api import ONE
+from one.api import ONE, One
+import one.alf.io as alfio
+from one.alf import cache
 
 from iblutil.util import Bunch
-
 from ibllib.io import spikeglx
 from ibllib.io.extractors.training_wheel import extract_wheel_moves, extract_first_movement_times
-from ibllib.ephys.neuropixel import SITES_COORDINATES, TIP_SIZE_UM
-from ibllib.atlas import atlas
-from ibllib.atlas import AllenAtlas
+from ibllib.ephys.neuropixel import SITES_COORDINATES, TIP_SIZE_UM, trace_header
+from ibllib.atlas import atlas, AllenAtlas
 from ibllib.pipes import histology
 from ibllib.pipes.ephys_alignment import EphysAlignment
 
 from brainbox.core import TimeSeries
 from brainbox.processing import sync
+from brainbox.metrics.single_units import quick_unit_metrics
 
 _logger = logging.getLogger('ibllib')
 
 
-SPIKES_ATTRIBUTES = ['clusters', 'times']
+SPIKES_ATTRIBUTES = ['clusters', 'times', 'amps', 'depths']
 CLUSTERS_ATTRIBUTES = ['channels', 'depths', 'metrics']
 
 
@@ -120,7 +123,7 @@ def _channels_alf2bunch(channels, brain_regions=None):
 
 
 def _load_spike_sorting(eid, one=None, collection=None, revision=None, return_channels=True, dataset_types=None,
-                        brain_regions=None):
+                        brain_regions=None, return_collection=False):
     """
     Generic function to load spike sorting according data using ONE.
 
@@ -143,7 +146,7 @@ def _load_spike_sorting(eid, one=None, collection=None, revision=None, return_ch
         A particular revision return (defaults to latest revision).  See `ALF documentation`_ for
         details.
     return_channels : bool
-        Defaults to False otherwise loads channels from disk (takes longer)
+        Defaults to False otherwise loads channels from disk
 
     .. _ALF documentation: https://one.internationalbrainlab.org/alf_intro.html#optional-components
 
@@ -208,9 +211,9 @@ def _load_channels_locations_from_disk(eid, collection=None, one=None, revision=
         # if the spike sorter has not aligned data, try and get the alignment available
         if 'brainLocationIds_ccf_2017' not in channels[probe].keys():
             aligned_channel_collections = one.list_collections(
-                eid, filename='channels.brainLocationIds_ccf_2017*', collection=f'alf/{probe}', revision=revision)
+                eid, filename='channels.brainLocationIds_ccf_2017*', collection=probe_collection, revision=revision)
             if len(aligned_channel_collections) == 0:
-                _logger.warning(f"no resolved alignment dataset found for {eid}/{probe}")
+                _logger.debug(f"no resolved alignment dataset found for {eid}/{probe}")
                 continue
             _logger.debug(f"looking for a resolved alignment dataset in {aligned_channel_collections}")
             ac_collection = _get_spike_sorting_collection(aligned_channel_collections, probe)
@@ -221,7 +224,7 @@ def _load_channels_locations_from_disk(eid, collection=None, one=None, revision=
     return channels
 
 
-def channel_locations_interpolation(channels_aligned, channels, brain_regions=None):
+def channel_locations_interpolation(channels_aligned, channels=None, brain_regions=None):
     """
     oftentimes the channel map for different spike sorters may be different so interpolate the alignment onto
     if there is no spike sorting in the base folder, the alignment doesn't have the localCoordinates field
@@ -238,6 +241,10 @@ def channel_locations_interpolation(channels_aligned, channels, brain_regions=No
       'x', 'y', 'z', 'acronym', 'atlas_id', 'axial_um', 'lateral_um'
     :return: Bunch or dictionary of channels with brain coordinates keys
     """
+    NEUROPIXEL_VERSION = 1
+    h = trace_header(version=NEUROPIXEL_VERSION)
+    if channels is None:
+        channels = {'localCoordinates': np.c_[h['x'], h['y']]}
     nch = channels['localCoordinates'].shape[0]
     if set(['x', 'y', 'z']).issubset(set(channels_aligned.keys())):
         channels_aligned = _channels_bunch2alf(channels_aligned)
@@ -245,9 +252,7 @@ def channel_locations_interpolation(channels_aligned, channels, brain_regions=No
         aligned_depths = channels_aligned['localCoordinates'][:, 1]
     else:  # this is a edge case for a few spike sorting sessions
         assert channels_aligned['mlapdv'].shape[0] == 384
-        NEUROPIXEL_VERSION = 1
-        from ibllib.ephys.neuropixel import trace_header
-        aligned_depths = trace_header(version=NEUROPIXEL_VERSION)['y']
+        aligned_depths = h['y']
     depth_aligned, ind_aligned = np.unique(aligned_depths, return_index=True)
     depths, ind, iinv = np.unique(channels['localCoordinates'][:, 1], return_index=True, return_inverse=True)
     channels['mlapdv'] = np.zeros((nch, 3))
@@ -264,8 +269,10 @@ def channel_locations_interpolation(channels_aligned, channels, brain_regions=No
 
 
 def _load_channel_locations_traj(eid, probe=None, one=None, revision=None, aligned=False,
-                                 brain_atlas=None):
-    print('from traj')
+                                 brain_atlas=None, return_source=False):
+    if not hasattr(one, 'alyx'):
+        return {}, None
+    _logger.debug(f"trying to load from traj {probe}")
     channels = Bunch()
     brain_atlas = brain_atlas or AllenAtlas
     # need to find the collection bruh
@@ -274,7 +281,6 @@ def _load_channel_locations_traj(eid, probe=None, one=None, revision=None, align
     collections = one.list_collections(eid, filename='channels*', collection=collection,
                                        revision=revision)
     probe_collection = _get_spike_sorting_collection(collections, probe)
-    print(probe_collection)
     chn_coords = one.load_dataset(eid, 'channels.localCoordinates', collection=probe_collection)
     depths = chn_coords[:, 1]
 
@@ -289,10 +295,9 @@ def _load_channel_locations_traj(eid, probe=None, one=None, revision=None, align
         xyz = np.array(insertion['json']['xyz_picks']) / 1e6
         if resolved:
 
-            _logger.info(f'Channel locations for {eid}/{probe} have been resolved. '
-                         f'Channel and cluster locations obtained from ephys aligned histology '
-                         f'track.')
-
+            _logger.debug(f'Channel locations for {eid}/{probe} have been resolved. '
+                          f'Channel and cluster locations obtained from ephys aligned histology '
+                          f'track.')
             traj = one.alyx.rest('trajectories', 'list', session=eid, probe=probe,
                                  provenance='Ephys aligned histology track')[0]
             align_key = insertion['json']['extended_qc']['alignment_stored']
@@ -303,12 +308,12 @@ def _load_channel_locations_traj(eid, probe=None, one=None, revision=None, align
                                         brain_atlas=brain_atlas, speedy=True)
             chans = ephysalign.get_channel_locations(feature, track)
             channels[probe] = _channels_traj2bunch(chans, brain_atlas)
-
+            source = 'resolved'
         elif counts > 0 and aligned:
-            _logger.info(f'Channel locations for {eid}/{probe} have not been '
-                         f'resolved. However, alignment flag set to True so channel and cluster'
-                         f' locations will be obtained from latest available ephys aligned '
-                         f'histology track.')
+            _logger.debug(f'Channel locations for {eid}/{probe} have not been '
+                          f'resolved. However, alignment flag set to True so channel and cluster'
+                          f' locations will be obtained from latest available ephys aligned '
+                          f'histology track.')
             # get the latest user aligned channels
             traj = one.alyx.rest('trajectories', 'list', session=eid, probe=probe,
                                  provenance='Ephys aligned histology track')[0]
@@ -321,28 +326,31 @@ def _load_channel_locations_traj(eid, probe=None, one=None, revision=None, align
             chans = ephysalign.get_channel_locations(feature, track)
 
             channels[probe] = _channels_traj2bunch(chans, brain_atlas)
-
+            source = 'aligned'
         else:
-            _logger.info(f'Channel locations for {eid}/{probe} have not been resolved. '
-                         f'Channel and cluster locations obtained from histology track.')
+            _logger.debug(f'Channel locations for {eid}/{probe} have not been resolved. '
+                          f'Channel and cluster locations obtained from histology track.')
             # get the channels from histology tracing
             xyz = xyz[np.argsort(xyz[:, 2]), :]
             chans = histology.interpolate_along_track(xyz, (depths + TIP_SIZE_UM) / 1e6)
 
             channels[probe] = _channels_traj2bunch(chans, brain_atlas)
-
+            source = 'traced'
         channels[probe]['axial_um'] = chn_coords[:, 1]
         channels[probe]['lateral_um'] = chn_coords[:, 0]
 
     else:
-        _logger.warning(f'Histology tracing for {probe} does not exist. '
-                        f'No channels for {probe}')
+        _logger.warning(f'Histology tracing for {probe} does not exist. No channels for {probe}')
+        source = ''
         channels = None
 
-    return channels
+    if return_source:
+        return channels, source
+    else:
+        return channels
 
 
-def load_channel_locations(eid, probe=None, one=None, aligned=False, brain_atlas=None):
+def load_channel_locations(eid, probe=None, one=None, aligned=False, brain_atlas=None, return_source=False):
     """
     Load the brain locations of each channel for a given session/probe
 
@@ -359,12 +367,14 @@ def load_channel_locations(eid, probe=None, one=None, aligned=False, brain_atlas
         Whether to get the latest user aligned channel when not resolved or use histology track
     brain_atlas : ibllib.atlas.BrainAtlas
         Brain atlas object (default: Allen atlas)
-
+    return_source: bool
+        if True returns the source of the channel lcoations (default False)
     Returns
     -------
     dict of one.alf.io.AlfBunch
         A dict with probe labels as keys, contains channel locations with keys ('acronym',
         'atlas_id', 'x', 'y', 'z').  Atlas IDs non-lateralized.
+    optional: string 'resolved', 'aligned', 'traced' or ''
     """
     one = one or ONE()
     brain_atlas = brain_atlas or AllenAtlas()
@@ -378,15 +388,15 @@ def load_channel_locations(eid, probe=None, one=None, aligned=False, brain_atlas
                                                   brain_regions=brain_atlas.regions)
     incomplete_probes = [k for k in channels if 'x' not in channels[k]]
     for iprobe in incomplete_probes:
-        channels_ = _load_channel_locations_traj(eid, probe=iprobe, one=one, aligned=aligned,
-                                                 brain_atlas=brain_atlas)
+        channels_, source = _load_channel_locations_traj(eid, probe=iprobe, one=one, aligned=aligned,
+                                                         brain_atlas=brain_atlas, return_source=True)
         if channels_ is not None:
             channels[iprobe] = channels_[iprobe]
     return channels
 
 
 def load_spike_sorting_fast(eid, one=None, probe=None, dataset_types=None, spike_sorter=None, revision=None,
-                            brain_regions=None, nested=True):
+                            brain_regions=None, nested=True, collection=None, return_collection=False):
     """
     From an eid, loads spikes and clusters for all probes
     The following set of dataset types are loaded:
@@ -401,27 +411,35 @@ def load_spike_sorting_fast(eid, one=None, probe=None, dataset_types=None, spike
     :param probe: name of probe to load in, if not given all probes for session will be loaded
     :param dataset_types: additional spikes/clusters objects to add to the standard default list
     :param spike_sorter: name of the spike sorting you want to load (None for default)
+    :param collection: name of the spike sorting collection to load - exclusive with spike sorter name ex: "alf/probe00"
     :param return_channels: (bool) defaults to False otherwise tries and load channels from disk
     :param brain_regions: ibllib.atlas.regions.BrainRegions object - will label acronyms if provided
     :param nested: if a single probe is required, do not output a dictionary with the probe name as key
-    :return: spikes, clusters (dict of bunch, 1 bunch per probe)
+    :param return_collection: (False) if True, will return the collection used to load
+    :return: spikes, clusters, channels (dict of bunch, 1 bunch per probe)
     """
-    collection = _collection_filter_from_args(probe, spike_sorter)
+    _logger.warning('Deprecation warning: brainbox.io.one.load_spike_sorting_fast will be removed in future versions.'
+                    'Use brainbox.io.one.SpikeSortingLoader instead')
+    if collection is None:
+        collection = _collection_filter_from_args(probe, spike_sorter)
     _logger.debug(f"load spike sorting with collection filter {collection}")
     kwargs = dict(eid=eid, one=one, collection=collection, revision=revision, dataset_types=dataset_types,
                   brain_regions=brain_regions)
     spikes, clusters, channels = _load_spike_sorting(**kwargs, return_channels=True)
     clusters = merge_clusters_channels(clusters, channels, keys_to_add_extra=None)
-    if nested is False:
+    if nested is False and len(spikes.keys()) == 1:
         k = list(spikes.keys())[0]
         channels = channels[k]
         clusters = clusters[k]
         spikes = spikes[k]
-    return spikes, clusters, channels
+    if return_collection:
+        return spikes, clusters, channels, collection
+    else:
+        return spikes, clusters, channels
 
 
 def load_spike_sorting(eid, one=None, probe=None, dataset_types=None, spike_sorter=None, revision=None,
-                       brain_regions=None):
+                       brain_regions=None, return_collection=False):
     """
     From an eid, loads spikes and clusters for all probes
     The following set of dataset types are loaded:
@@ -438,18 +456,24 @@ def load_spike_sorting(eid, one=None, probe=None, dataset_types=None, spike_sort
     :param spike_sorter: name of the spike sorting you want to load (None for default)
     :param return_channels: (bool) defaults to False otherwise tries and load channels from disk
     :param brain_regions: ibllib.atlas.regions.BrainRegions object - will label acronyms if provided
+    :param return_collection:(bool - False) if True, returns the collection for loading the data
     :return: spikes, clusters (dict of bunch, 1 bunch per probe)
     """
+    _logger.warning('Deprecation warning: brainbox.io.one.load_spike_sorting will be removed in future versions.'
+                    'Use brainbox.io.one.SpikeSortingLoader instead')
     collection = _collection_filter_from_args(probe, spike_sorter)
     _logger.debug(f"load spike sorting with collection filter {collection}")
     spikes, clusters = _load_spike_sorting(eid=eid, one=one, collection=collection, revision=revision,
                                            return_channels=False, dataset_types=dataset_types,
                                            brain_regions=brain_regions)
-    return spikes, clusters
+    if return_collection:
+        return spikes, clusters, collection
+    else:
+        return spikes, clusters
 
 
 def load_spike_sorting_with_channel(eid, one=None, probe=None, aligned=False, dataset_types=None,
-                                    spike_sorter=None, brain_atlas=None):
+                                    spike_sorter=None, brain_atlas=None, nested=True, return_collection=False):
     """
     For a given eid, get spikes, clusters and channels information, and merges clusters
     and channels information before returning all three variables.
@@ -472,6 +496,8 @@ def load_spike_sorting_with_channel(eid, one=None, probe=None, aligned=False, da
         available otherwise the default MATLAB kilosort)
     brain_atlas : ibllib.atlas.BrainAtlas
         Brain atlas object (default: Allen atlas)
+    return_collection: bool
+        Returns an extra argument with the collection chosen
 
     Returns
     -------
@@ -486,15 +512,25 @@ def load_spike_sorting_with_channel(eid, one=None, probe=None, aligned=False, da
         'atlas_id', 'x', 'y', 'z').  Atlas IDs non-lateralized.
     """
     # --- Get spikes and clusters data
+    _logger.warning('Deprecation warning: brainbox.io.one.load_spike_sorting will be removed in future versions.'
+                    'Use brainbox.io.one.SpikeSortingLoader instead')
     one = one or ONE()
     brain_atlas = brain_atlas or AllenAtlas()
-    spikes, clusters = load_spike_sorting(eid, one=one, probe=probe, dataset_types=dataset_types,
-                                          spike_sorter=spike_sorter)
+    spikes, clusters, collection = load_spike_sorting(
+        eid, one=one, probe=probe, dataset_types=dataset_types, spike_sorter=spike_sorter, return_collection=True)
     # -- Get brain regions and assign to clusters
     channels = load_channel_locations(eid, one=one, probe=probe, aligned=aligned,
                                       brain_atlas=brain_atlas)
     clusters = merge_clusters_channels(clusters, channels, keys_to_add_extra=None)
-    return spikes, clusters, channels
+    if nested is False and len(spikes.keys()) == 1:
+        k = list(spikes.keys())[0]
+        channels = channels[k]
+        clusters = clusters[k]
+        spikes = spikes[k]
+    if return_collection:
+        return spikes, clusters, channels, collection
+    else:
+        return spikes, clusters, channels
 
 
 def load_ephys_session(eid, one=None):
@@ -830,3 +866,170 @@ def load_channels_from_insertion(ins, depths=None, one=None, ba=None):
                                         brain_atlas=ba, speedy=True)
             xyz_channels = ephysalign.get_channel_locations(feature, track)
     return xyz_channels
+
+
+@dataclass
+class SpikeSortingLoader:
+    """
+    Object that will load spike sorting data for a given probe insertion.
+    This class can be instantiated in several manners
+    - With Alyx database probe id:
+            SpikeSortingLoader(pid=pid, one=one)
+    - With Alyx database eic and probe name:
+            SpikeSortingLoader(eid=eid, pname='probe00', one=one)
+    - From a local session and probe name:
+            SpikeSortingLoader(session_path=session_path, pname='probe00')
+    """
+    one: ONE = None
+    atlas: None = None
+    pid: str = None
+    eid: str = ''
+    pname: str = ''
+    # the following properties are the outcome of the post init funciton
+    session_path: Path = ''
+    collections: list = None
+    datasets: list = None   # list of all datasets belonging to the sesion
+    # the following properties are the outcome of a reading function
+    files: dict = None
+    collection: str = ''
+    histology: str = ''  # 'alf', 'resolved', 'aligned' or 'traced'
+    spike_sorting_path: Path = None
+
+    def __post_init__(self):
+        # pid gets precedence
+        if self.pid is not None:
+            self.eid, self.pname = self.one.pid2eid(self.pid)
+            self.session_path = self.one.eid2path(self.eid)
+        # then eid / pname combination
+        elif self.session_path is None or self.session_path == '':
+            self.session_path = self.one.eid2path(self.eid)
+        # fully local providing a session path
+        else:
+            self.one = One(cache_dir=self.session_path.parents[2], mode='local')
+            df_sessions = cache._make_sessions_df(self.session_path)
+            self.one._cache['sessions'] = df_sessions.set_index('id')
+            self.one._cache['datasets'] = cache._make_datasets_df(self.session_path, hash_files=False)
+            self.eid = str(self.session_path.relative_to(self.session_path.parents[2]))
+        # populates default properties
+        self.collections = self.one.list_collections(
+            self.eid, filename='spikes*', collection=f"alf/{self.pname}*")
+        self.datasets = self.one.list_datasets(self.eid)
+        if self.atlas is None:
+            self.atlas = AllenAtlas()
+        self.files = {}
+
+    @staticmethod
+    def _get_attributes(dataset_types):
+        """returns attributes to load for spikes and clusters objects"""
+        if dataset_types is None:
+            return SPIKES_ATTRIBUTES, CLUSTERS_ATTRIBUTES
+        else:
+            spike_attributes = [sp.split('.')[1] for sp in dataset_types if 'spikes.' in sp]
+            cluster_attributes = [cl.split('.')[1] for cl in dataset_types if 'clusters.' in cl]
+            spike_attributes = list(set(SPIKES_ATTRIBUTES + spike_attributes))
+            cluster_attributes = list(set(CLUSTERS_ATTRIBUTES + cluster_attributes))
+            return spike_attributes, cluster_attributes
+
+    def _get_spike_sorting_collection(self, spike_sorter='pykilosort', revision=None):
+        """
+        Filters a list or array of collections to get the relevant spike sorting dataset
+        if there is a pykilosort, load it
+        """
+        collection = next(filter(lambda c: c == f'alf/{self.pname}/{spike_sorter}', self.collections), None)
+        # otherwise, prefers the shortest
+        collection = collection or next(iter(sorted(filter(lambda c: f'alf/{self.pname}' in c, self.collections), key=len)), None)
+        _logger.debug(f"selecting: {collection} to load amongst candidates: {self.collections}")
+        return collection
+
+    def _download_spike_sorting_object(self, obj, spike_sorter='pykilosort', dataset_types=None):
+        """
+        Downloads an ALF object
+        :param obj: object name, str between 'spikes', 'clusters' or 'channels'
+        :param spike_sorter: (defaults to 'pykilosort')
+        :param dataset_types: list of extra dataset types
+        :return:
+        """
+        if len(self.collections) == 0:
+            return {}, {}, {}
+        self.collection = self._get_spike_sorting_collection(spike_sorter=spike_sorter)
+        _logger.debug(f"loading spike sorting from {self.collection}")
+        spike_attributes, cluster_attributes = self._get_attributes(dataset_types)
+        attributes = {'spikes': spike_attributes, 'clusters': cluster_attributes, 'channels': None}
+        self.files[obj] = self.one.load_object(self.eid, obj=obj, attribute=attributes[obj],
+                                               collection=self.collection, download_only=True)
+
+    def download_spike_sorting(self, **kwargs):
+        """
+        Downloads spikes, clusters and channels
+        :param spike_sorter: (defaults to 'pykilosort')
+        :param dataset_types: list of extra dataset types
+        :return:
+        """
+        for obj in ['spikes', 'clusters', 'channels']:
+            self._download_spike_sorting_object(obj=obj, **kwargs)
+        self.spike_sorting_path = self.files['spikes'][0].parent
+
+    def load_spike_sorting(self, **kwargs):
+        """
+        Loads spikes, clusters and channels
+
+        There could be several spike sorting collections, by default the loader will get the pykilosort collection
+
+        The channel locations can come from several sources, it will load the most advanced version of the histology available,
+        regardless of the spike sorting version loaded. The steps are (from most advanced to fresh out of the imaging):
+        -   alf: the final version of channel locations, same as resolved with the difference that data is on file
+        -   resolved: channel locations alignments have been agreed upon
+        -   aligned: channel locations have been aligned, but review or other alignments are pending, potentially not accurate
+        -   traced: the histology track has been recovered from microscopy, however the depths may not match, inacurate data
+
+        :param spike_sorter: (defaults to 'pykilosort')
+        :param dataset_types: list of extra dataset types
+        :return:
+        """
+        if len(self.collections) == 0:
+            return {}, {}, {}
+        self.download_spike_sorting(**kwargs)
+        channels = alfio.load_object(self.files['channels'], wildcards=self.one.wildcards)
+        clusters = alfio.load_object(self.files['clusters'], wildcards=self.one.wildcards)
+        spikes = alfio.load_object(self.files['spikes'], wildcards=self.one.wildcards)
+        if 'brainLocationIds_ccf_2017' not in channels:
+            _channels, self.histology = _load_channel_locations_traj(
+                self.eid, probe=self.pname, one=self.one, brain_atlas=self.atlas, return_source=True)
+            if _channels:
+                channels = _channels[self.pname]
+        else:
+            channels = _channels_alf2bunch(channels, brain_regions=self.atlas.regions)
+            self.histology = 'alf'
+        return spikes, clusters, channels
+
+    @staticmethod
+    def merge_clusters(spikes, clusters, channels, cache_dir=None):
+        """merge metrics and channels info - optionally saves a clusters.pqt dataframe"""
+        if spikes == {}:
+            return
+        nc = clusters['channels'].size
+        # recompute metrics if they are not available
+        metrics = None
+        if 'metrics' in clusters:
+            metrics = clusters.pop('metrics')
+            if metrics.shape[0] != nc:
+                metrics = None
+        if metrics is None:
+            _logger.debug("recompute clusters metrics")
+            metrics = pd.DataFrame(quick_unit_metrics(
+                spikes['clusters'], spikes['times'], spikes['amps'], spikes['depths'], cluster_ids=np.arange(nc)))
+            if isinstance(cache_dir, Path):
+                metrics.to_parquet(Path(cache_dir).joinpath('clusters.metrics.pqt'))
+        for k in metrics.keys():
+            clusters[k] = metrics[k].to_numpy()
+
+        for k in channels.keys():
+            clusters[k] = channels[k][clusters['channels']]
+        if cache_dir:
+            pd.DataFrame(clusters).to_parquet(Path(cache_dir).joinpath('clusters.pqt'))
+        return clusters
+
+    @property
+    def url(self):
+        """Gets flatiron URL for the session"""
+        return str(self.session_path).replace(str(self.one.alyx.cache_dir), 'https://ibl.flatironinstitute.org')
