@@ -109,6 +109,37 @@ def fk(x, si=.002, dx=1, vbounds=None, btype='highpass', ntr_pad=0, ntr_tap=None
     return xf / gain
 
 
+def car(x, collection=None, lagc=300, butter_kwargs=None, **kwargs):
+    """
+    Applies common average referencing with optional automatic gain control
+    :param x: the input array to be filtered. dimension, the filtering is considering
+    axis=0: spatial dimension, axis=1 temporal dimension. (ntraces, ns)
+    :param collection:
+    :param lagc: window size for time domain automatic gain control (no agc otherwise)
+    :param butter_kwargs: filtering parameters: defaults: {'N': 3, 'Wn': 0.1, 'btype': 'highpass'}
+    :return:
+    """
+    if butter_kwargs is None:
+        butter_kwargs = {'N': 3, 'Wn': 0.1, 'btype': 'highpass'}
+    if collection is not None:
+        xout = np.zeros_like(x)
+        for c in np.unique(collection):
+            sel = collection == c
+            xout[sel, :] = kfilt(x=x[sel, :], ntr_pad=0, ntr_tap=None, collection=None,
+                                 butter_kwargs=butter_kwargs)
+        return xout
+
+    # apply agc and keep the gain in handy
+    if not lagc:
+        xf = np.copy(x)
+        gain = 1
+    else:
+        xf, gain = agc(x, wl=lagc, si=1.0)
+    # apply CAR and then un-apply the gain
+    xf = xf - np.median(xf, axis=0)
+    return xf / gain
+
+
 def kfilt(x, collection=None, ntr_pad=0, ntr_tap=None, lagc=300, butter_kwargs=None):
     """
     Applies a butterworth filter on the 0-axis with tapering / padding
@@ -192,7 +223,22 @@ def interpolate_bad_channels(data, channel_labels=None, h=None, p=1.3, kriging_d
     return data
 
 
-def destripe(x, fs, neuropixel_version=1, butter_kwargs=None, k_kwargs=None, channel_labels=None):
+def _get_destripe_parameters(fs, butter_kwargs, k_kwargs, k_filter):
+    """gets the default params for destripe. This is used for both the destripe fcn on a
+    numpy array and the function that actuates on a cbin file"""
+    if butter_kwargs is None:
+        butter_kwargs = {'N': 3, 'Wn': 300 / fs * 2, 'btype': 'highpass'}
+    if k_kwargs is None:
+        k_kwargs = {'ntr_pad': 60, 'ntr_tap': 0, 'lagc': 3000,
+                    'butter_kwargs': {'N': 3, 'Wn': 0.01, 'btype': 'highpass'}}
+    if k_filter:
+        spatial_fcn = lambda dat: kfilt(dat, **k_kwargs)  # noqa
+    else:
+        spatial_fcn = lambda dat: car(dat, **k_kwargs)  # noqa
+    return butter_kwargs, k_kwargs, spatial_fcn
+
+
+def destripe(x, fs, neuropixel_version=1, butter_kwargs=None, k_kwargs=None, channel_labels=None, k_filter=True):
     """Super Car (super slow also...) - far from being set in stone but a good workflow example
     :param x: demultiplexed array (nc, ns)
     :param fs: sampling frequency
@@ -209,13 +255,11 @@ def destripe(x, fs, neuropixel_version=1, butter_kwargs=None, k_kwargs=None, cha
        True: deduces the bad channels from the data provided
     :param butter_kwargs: (optional, None) butterworth params, see the code for the defaults dict
     :param k_kwargs: (optional, None) K-filter params, see the code for the defaults dict
+        can also be set to 'car', in which case the median accross channels will be subtracted
+    :param k_filter (True): applies k-filter by default, otherwise, apply CAR.
     :return: x, filtered array
     """
-    if butter_kwargs is None:
-        butter_kwargs = {'N': 3, 'Wn': 300 / fs * 2, 'btype': 'highpass'}
-    if k_kwargs is None:
-        k_kwargs = {'ntr_pad': 60, 'ntr_tap': 0, 'lagc': 3000,
-                    'butter_kwargs': {'N': 3, 'Wn': 0.01, 'btype': 'highpass'}}
+    butter_kwargs, k_kwargs, spatial_fcn = _get_destripe_parameters(fs, butter_kwargs, k_kwargs, k_filter)
     h = neuropixel.trace_header(version=neuropixel_version)
     if channel_labels is True:
         channel_labels, _ = detect_bad_channels(x, fs)
@@ -225,27 +269,42 @@ def destripe(x, fs, neuropixel_version=1, butter_kwargs=None, k_kwargs=None, cha
     # channel interpolation
     # apply ADC shift
     if neuropixel_version is not None:
-        sample_shift = h['sample_shift'] if (30000 / fs) < 10 else h['sample_shift'] * fs / 30000
-        x = fshift(x, sample_shift, axis=1)
+        x = fshift(x, h['sample_shift'], axis=1)
     # apply spatial filter only on channels that are inside of the brain
     if channel_labels is not None:
         x = interpolate_bad_channels(x, channel_labels, h)
         inside_brain = np.where(channel_labels != 3)[0]
-        x[inside_brain, :] = kfilt(x[inside_brain, :], **k_kwargs)  # apply the k-filter
+        x[inside_brain, :] = spatial_fcn(x[inside_brain, :])  # apply the k-filter
     else:
-        x = kfilt(x, **k_kwargs)
+        x = spatial_fcn(x)
     return x
 
 
+def destripe_lfp(x, fs, channel_labels=None, **kwargs):
+    """
+    Wrapper around the destipe function with some default parameters to destripe the LFP band
+    See help destripe function for documentation
+    :param x:
+    :param fs:
+    :return:
+    """
+    kwargs['butter_kwargs'] = {'N': 3, 'Wn': 2 / fs * 2, 'btype': 'highpass'}
+    kwargs['k_filter'] = False
+    if channel_labels is True:
+        kwargs['channel_labels'], _ = detect_bad_channels(x, fs=fs, psd_hf_threshold=1.4)
+    return destripe(x, fs, **kwargs)
+
+
 def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, append=False, nc_out=None, butter_kwargs=None,
-                             dtype=np.int16, ns2add=0, nbatch=None, nprocesses=None, compute_rms=True, reject_channels=True):
+                             dtype=np.int16, ns2add=0, nbatch=None, nprocesses=None, compute_rms=True, reject_channels=True,
+                             k_kwargs=None, k_filter=True, reader_kwargs=None):
     """
     From a spikeglx Reader object, decompresses and apply ADC.
     Saves output as a flat binary file in int16
     Production version with optimized FFTs - requires pyfftw
     :param sr: seismic reader object (spikeglx.Reader)
     :param output_file: (optional, defaults to .bin extension of the compressed bin file)
-    :param h: (optional)
+    :param h: (optional) neuropixel trace header. Dictionary with key 'sample_shift'
     :param wrot: (optional) whitening matrix [nc x nc] or amplitude scalar to apply to the output
     :param append: (optional, False) for chronic recordings, append to end of file
     :param nc_out: (optional, True) saves non selected channels (synchronisation trace) in output
@@ -258,6 +317,9 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
      interp 3:outside of brain and discard
     :param reject_channels: (True) detects noisy or bad channels and interpolate them. Channels outside of the brain are left
      untouched
+    :param k_kwargs: (None) arguments for the kfilter function
+    :param reader_kwargs: (None) optional arguments for the spikeglx Reader instance
+    :param k_filter: (True) Performs a k-filter - if False will do median common average referencing
     :return:
     """
     import pyfftw
@@ -265,15 +327,14 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
     SAMPLES_TAPER = 1024
     NBATCH = nbatch or 65536
     # handles input parameters
-    sr = spikeglx.Reader(sr_file, open=True)
+    reader_kwargs = {} if reader_kwargs is None else reader_kwargs
+    sr = spikeglx.Reader(sr_file, open=True, **reader_kwargs)
     if reject_channels:  # get bad channels if option is on
         channel_labels = detect_bad_channels_cbin(sr)
     assert isinstance(sr_file, str) or isinstance(sr_file, Path)
-    butter_kwargs = butter_kwargs or {'N': 3, 'Wn': 300 / sr.fs * 2, 'btype': 'highpass'}
-    k_kwargs = {'ntr_pad': 60, 'ntr_tap': 0, 'lagc': 3000,
-                'butter_kwargs': {'N': 3, 'Wn': 0.01, 'btype': 'highpass'}}
-    h = neuropixel.trace_header(version=1) if h is None else h
-    ncv = h['x'].size  # number of channels
+    butter_kwargs, k_kwargs, spatial_fcn = _get_destripe_parameters(sr.fs, butter_kwargs, k_kwargs, k_filter)
+    h = sr.geometry if h is None else h
+    ncv = h['sample_shift'].size  # number of channels
     output_file = sr.file_bin.with_suffix('.bin') if output_file is None else output_file
     assert output_file != sr.file_bin
     taper = np.r_[0, scipy.signal.windows.cosine((SAMPLES_TAPER - 1) * 2), 0]
@@ -321,7 +382,7 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
     CHUNK_SIZE = int(sr.ns / nprocesses)
 
     def my_function(i_chunk, n_chunk):
-        _sr = spikeglx.Reader(sr_file)
+        _sr = spikeglx.Reader(sr_file, **reader_kwargs)
 
         n_batch = int(np.ceil(i_chunk * CHUNK_SIZE / NBATCH))
         first_s = (NBATCH - SAMPLES_TAPER * 2) * n_batch
@@ -375,19 +436,23 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
             if reject_channels:
                 chunk = interpolate_bad_channels(chunk, channel_labels, h=h)
                 inside_brain = np.where(channel_labels != 3)[0]
-                chunk[inside_brain, :] = kfilt(chunk[inside_brain, :], **k_kwargs)  # apply the k-filter
+                chunk[inside_brain, :] = spatial_fcn(chunk[inside_brain, :])  # apply the k-filter / CAR
             else:
-                chunk = kfilt(chunk, **k_kwargs)  # apply the k-filter
+                chunk = spatial_fcn(chunk)  # apply the k-filter / CAR
+
             # add back sync trace and save
             chunk = np.r_[chunk, _sr[first_s:last_s, ncv:].T].T
-            intnorm = 1 / _sr.channel_conversion_sample2v['ap'] if dtype == np.int16 else 1.
-            chunk = chunk[slice(*ind2save), :] * intnorm
+
             # Compute rms - we get it before applying the whitening
             if compute_rms:
                 ap_rms = rms(chunk[:, :ncv], axis=0)
                 ap_t = t0 + (first_s + (last_s - first_s - 1) / 2) / _sr.fs
                 ap_rms.astype(np.float32).tofile(aid)
                 ap_t.astype(np.float32).tofile(tid)
+
+            # convert to normalised
+            intnorm = 1 / _sr.sample2volts
+            chunk = chunk[slice(*ind2save), :] * intnorm
             # apply the whitening matrix if necessary
             if wrot is not None:
                 chunk[:, :ncv] = np.dot(chunk[:, :ncv], wrot)
@@ -439,7 +504,7 @@ def rcoeff(x, y):
     return rcor
 
 
-def detect_bad_channels(raw, fs, similarity_threshold=(-0.5, 1), psd_hf_threshold=0.02):
+def detect_bad_channels(raw, fs, similarity_threshold=(-0.5, 1), psd_hf_threshold=None):
     """
     Bad channels detection for Neuropixel probes
     Labels channels
@@ -503,8 +568,10 @@ def detect_bad_channels(raw, fs, similarity_threshold=(-0.5, 1), psd_hf_threshol
     raw = raw - np.mean(raw, axis=-1)[:, np.newaxis]  # removes DC offset
     xcor = channels_similarity(raw)
     fscale, psd = scipy.signal.welch(raw * 1e6, fs=fs)  # units; uV ** 2 / Hz
-
-    sos_hp = scipy.signal.butter(**{'N': 3, 'Wn': 1000 / fs / 2, 'btype': 'highpass'}, output='sos')
+    if psd_hf_threshold is None:
+        # the LFP band data is obviously much stronger so auto-adjust the default threshold
+        psd_hf_threshold = 1.4 if fs < 5000 else 0.02
+    sos_hp = scipy.signal.butter(**{'N': 3, 'Wn': 300 / fs * 2, 'btype': 'highpass'}, output='sos')
     hf = scipy.signal.sosfiltfilt(sos_hp, raw)
     xcorf = channels_similarity(hf)
 
@@ -513,7 +580,7 @@ def detect_bad_channels(raw, fs, similarity_threshold=(-0.5, 1), psd_hf_threshol
         'rms_raw': rms(raw),  # very similar to the rms avfter butterworth filter
         'xcor_hf': detrend(xcor, 11),
         'xcor_lf': xcorf - detrend(xcorf, 11) - 1,
-        'psd_hf': np.mean(psd[:, fscale > 12000], axis=-1),
+        'psd_hf': np.mean(psd[:, fscale > (fs / 2 * 0.8)], axis=-1),  # 80% nyquists
     })
 
     # make recommendation
