@@ -127,6 +127,7 @@ class ServerGlobusDataHandler(DataHandler):
 
         # Find the lab
         labs = get_lab_from_endpoint_id(one=self.one)
+
         if len(labs) == 2:
             # for flofer lab
             subject = self.one.path2ref(self.session_path)['subject']
@@ -167,13 +168,19 @@ class ServerGlobusDataHandler(DataHandler):
         target_paths = []
         source_paths = []
         self.local_paths = []
-        for _, d in df.iterrows():
+        for i, d in df.iterrows():
             sess_path = Path(rel_sess_path).joinpath(d['rel_path'])
             full_local_path = Path(self.globus.endpoints['local']['root_path']).joinpath(sess_path)
             if not full_local_path.exists():
+
+                if self.one._index_type() is int:
+                    uuid = np2str(np.r_[i[0], i[1]])
+                elif self.one._index_type() is str:
+                    uuid = i
+
                 self.local_paths.append(full_local_path)
                 target_paths.append(sess_path)
-                source_paths.append(add_uuid_string(sess_path, np2str(np.r_[d.name[0], d.name[1]])))
+                source_paths.append(add_uuid_string(sess_path, uuid))
 
         if len(target_paths) != 0:
             ts = time()
@@ -384,9 +391,15 @@ class SDSCDataHandler(DataHandler):
         df = super().getData()
 
         SDSC_TMP = Path(SDSC_PATCH_PATH.joinpath(self.task.__class__.__name__))
-        for _, d in df.iterrows():
+        for i, d in df.iterrows():
             file_path = Path(d['session_path']).joinpath(d['rel_path'])
-            file_uuid = add_uuid_string(file_path, np2str(np.r_[d.name[0], d.name[1]]))
+
+            if self.one._index_type() is int:
+                uuid = np2str(np.r_[i[0], i[1]])
+            elif self.one._index_type() is str:
+                uuid = i
+
+            file_uuid = add_uuid_string(file_path, uuid)
             file_link = SDSC_TMP.joinpath(file_path)
             file_link.parent.mkdir(exist_ok=True, parents=True)
             file_link.symlink_to(
