@@ -7,23 +7,29 @@ import traceback
 from pathlib import Path
 import packaging.version
 
+import cv2
 import numpy as np
 import pandas as pd
+
 import one.alf.io as alfio
 
 from ibllib.misc import check_nvidia_driver
 from ibllib.ephys import ephysqc, spikes, sync_probes
 from ibllib.io import ffmpeg, spikeglx
-from ibllib.io.video import label_from_path
+from ibllib.io.video import label_from_path, assert_valid_label
 from ibllib.io.extractors import ephys_fpga, ephys_passive, camera
 from ibllib.pipes import tasks
 from ibllib.pipes.training_preprocessing import TrainingRegisterRaw as EphysRegisterRaw
 from ibllib.pipes.misc import create_alyx_probe_insertions
+from ibllib.qc.alignment_qc import get_aligned_channels
 from ibllib.qc.task_extractors import TaskQCExtractor
 from ibllib.qc.task_metrics import TaskQC
 from ibllib.qc.camera import run_all_qc as run_camera_qc
 from ibllib.qc.dlc import DlcQC
 from ibllib.dsp import rms
+from ibllib.plots.figures import dlc_qc_plot, BehaviourPlots, LfpPlots, ApPlots, BadChannelsAp
+from ibllib.plots.figures import SpikeSorting as SpikeSortingPlots
+from ibllib.plots.snapshot import ReportSnapshot
 from brainbox.behavior.dlc import likelihood_threshold, get_licks, get_pupil_diameter, get_smooth_pupil_diameter
 
 _logger = logging.getLogger("ibllib")
@@ -128,6 +134,14 @@ class RawEphysQC(tasks.Task):
             try:
                 eqc = ephysqc.EphysQC(pid, session_path=self.session_path, one=self.one)
                 qc_files.extend(eqc.run(update=True, overwrite=overwrite))
+                _logger.info("Creating LFP QC plots")
+                plot_task = LfpPlots(pid, session_path=self.session_path, one=self.one)
+                _ = plot_task.run()
+                self.plot_tasks.append(plot_task)
+                plot_task = BadChannelsAp(pid, session_path=self.session_path, one=self.one)
+                _ = plot_task.run()
+                self.plot_tasks.append(plot_task)
+
             except AssertionError:
                 _logger.error(traceback.format_exc())
                 self.status = -1
@@ -403,6 +417,27 @@ class SpikeSorting(tasks.Task):
                 tar_dir.mkdir(parents=True, exist_ok=True)
                 out = spikes.ks2_to_tar(ks2_dir, tar_dir, force=self.FORCE_RERUN)
                 out_files.extend(out)
+
+                if self.one:
+                    eid = self.one.path2eid(self.session_path, query_type='remote')
+                    ins = self.one.alyx.rest('insertions', 'list', session=eid, name=label, query_type='remote')
+                    if len(ins) != 0:
+                        _logger.info("Creating SpikeSorting QC plots")
+                        plot_task = ApPlots(ins[0]['id'], session_path=self.session_path, one=self.one)
+                        _ = plot_task.run()
+                        self.plot_tasks.append(plot_task)
+
+                        plot_task = SpikeSortingPlots(ins[0]['id'], session_path=self.session_path, one=self.one)
+                        _ = plot_task.run(collection=str(probe_out_path.relative_to(self.session_path)))
+                        self.plot_tasks.append(plot_task)
+
+                        resolved = ins[0].get('json', {'temp': 0}).get('extended_qc', {'temp': 0}). \
+                            get('alignment_resolved', False)
+                        if resolved:
+                            chns = np.load(probe_out_path.joinpath('channels.localCoordinates.npy'))
+                            out = get_aligned_channels(ins[0], chns, one=self.one, save_dir=probe_out_path)
+                            out_files.extend(out)
+
             except BaseException:
                 _logger.error(traceback.format_exc())
                 self.status = -1
@@ -620,7 +655,7 @@ class EphysTrials(tasks.Task):
             "sessions", eid, "extended_qc", {"behavior": int(good_enough)}
         )
 
-    def _run(self):
+    def _run(self, plot_qc=True):
         dsets, out_files = ephys_fpga.extract_all(self.session_path, save=True)
 
         if not self.one or self.one.offline:
@@ -635,6 +670,20 @@ class EphysTrials(tasks.Task):
         qc.extractor.extract_data()
         # Aggregate and update Alyx QC fields
         qc.run(update=True)
+
+        if plot_qc:
+            _logger.info("Creating Trials QC plots")
+            try:
+                session_id = self.one.path2eid(self.session_path)
+                plot_task = BehaviourPlots(session_id, self.session_path, one=self.one)
+                _ = plot_task.run()
+                self.plot_tasks.append(plot_task)
+
+            except BaseException:
+                _logger.error('Could not create Trials QC Plot')
+                _logger.error(traceback.format_exc())
+                self.status = -1
+
         return out_files
 
     def get_signatures(self, **kwargs):
@@ -848,14 +897,159 @@ class EphysMtscomp(tasks.Task):
 
 
 class EphysDLC(tasks.Task):
+    """
+    This task relies on a correctly installed dlc environment as per
+    https://docs.google.com/document/d/1g0scP6_3EmaXCU4SsDNZWwDTaD9MG0es_grLA-d0gh0/edit#
+
+    If your environment is set up otherwise, make sure that you set the respective attributes:
+    t = EphysDLC(session_path)
+    t.dlcenv = Path('/path/to/your/dlcenv/bin/activate')
+    t.scripts = Path('/path/to/your/iblscripts/deploy/serverpc/dlc')
+    """
     gpu = 1
     cpu = 4
     io_charge = 90
     level = 2
+    force = True
 
-    def _run(self):
-        """empty placeholder for job creation only"""
-        pass
+    dlcenv = Path.home().joinpath('Documents', 'PYTHON', 'envs', 'dlcenv', 'bin', 'activate')
+    scripts = Path.home().joinpath('Documents', 'PYTHON', 'iblscripts', 'deploy', 'serverpc', 'dlc')
+    signature = {
+        'input_files': [
+            ('_iblrig_leftCamera.raw.mp4', 'raw_video_data', True),
+            ('_iblrig_rightCamera.raw.mp4', 'raw_video_data', True),
+            ('_iblrig_bodyCamera.raw.mp4', 'raw_video_data', True),
+        ],
+        'output_files': [
+            ('_ibl_leftCamera.dlc.pqt', 'alf', True),
+            ('_ibl_rightCamera.dlc.pqt', 'alf', True),
+            ('_ibl_bodyCamera.dlc.pqt', 'alf', True),
+            ('leftCamera.ROIMotionEnergy.npy', 'alf', True),
+            ('rightCamera.ROIMotionEnergy.npy', 'alf', True),
+            ('bodyCamera.ROIMotionEnergy.npy', 'alf', True),
+            ('leftROIMotionEnergy.position.npy', 'alf', True),
+            ('rightROIMotionEnergy.position.npy', 'alf', True),
+            ('bodyROIMotionEnergy.position.npy', 'alf', True),
+        ],
+    }
+
+    def _check_dlcenv(self):
+        """Check that scripts are present, dlcenv can be activated and get iblvideo version"""
+        assert len(list(self.scripts.rglob('run_dlc.*'))) == 2, \
+            f'Scripts run_dlc.sh and run_dlc.py do not exist in {self.scripts}'
+        assert len(list(self.scripts.rglob('run_motion.*'))) == 2, \
+            f'Scripts run_motion.sh and run_motion.py do not exist in {self.scripts}'
+        assert self.dlcenv.exists(), f"DLC environment does not exist in assumed location {self.dlcenv}"
+        command2run = f"source {self.dlcenv}; python -c 'import iblvideo; print(iblvideo.__version__)'"
+        process = subprocess.Popen(
+            command2run,
+            shell=True,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            executable="/bin/bash"
+        )
+        info, error = process.communicate()
+        if process.returncode != 0:
+            raise AssertionError(f"DLC environment check failed\n{error.decode('utf-8')}")
+        version = info.decode("utf-8").strip().split('\n')[-1]
+        return version
+
+    @staticmethod
+    def _video_intact(file_mp4):
+        """Checks that the downloaded video can be opened and is not empty"""
+        cap = cv2.VideoCapture(str(file_mp4))
+        frame_count = cap.get(cv2.CAP_PROP_FRAME_COUNT)
+        intact = True if frame_count > 0 else False
+        cap.release()
+        return intact
+
+    def _run(self, cams=None, overwrite=False):
+        # Default to all three cams
+        cams = cams or ['left', 'right', 'body']
+        cams = assert_valid_label(cams)
+        # Set up
+        self.session_id = self.one.path2eid(self.session_path)
+        actual_outputs = []
+
+        # Loop through cams
+        for cam in cams:
+            # Catch exceptions so that following cameras can still run
+            try:
+                # If all results exist and overwrite is False, skip computation
+                expected_outputs_present, expected_outputs = self.assert_expected(self.output_files, silent=True)
+                if overwrite is False and expected_outputs_present is True:
+                    actual_outputs.extend(expected_outputs)
+                    return actual_outputs
+                else:
+                    file_mp4 = next(self.session_path.joinpath('raw_video_data').glob(f'_iblrig_{cam}Camera.raw*.mp4'))
+                    if not file_mp4.exists():
+                        # In this case we set the status to Incomplete.
+                        _logger.error(f"No raw video file available for {cam}, skipping.")
+                        self.status = -3
+                        continue
+                    if not self._video_intact(file_mp4):
+                        _logger.error(f"Corrupt raw video file {file_mp4}")
+                        self.status = -1
+                        continue
+                    # Check that dlc environment is ok, shell scripts exists, and get iblvideo version, GPU addressable
+                    self.version = self._check_dlcenv()
+                    _logger.info(f'iblvideo version {self.version}')
+                    check_nvidia_driver()
+
+                    _logger.info(f'Running DLC on {cam}Camera.')
+                    command2run = f"{self.scripts.joinpath('run_dlc.sh')} {str(self.dlcenv)} {file_mp4} {overwrite}"
+                    _logger.info(command2run)
+                    process = subprocess.Popen(
+                        command2run,
+                        shell=True,
+                        stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE,
+                        executable="/bin/bash",
+                    )
+                    info, error = process.communicate()
+                    info_str = info.decode("utf-8").strip()
+                    _logger.info(info_str)
+                    if process.returncode != 0:
+                        error_str = error.decode("utf-8").strip()
+                        _logger.error(f'DLC failed for {cam}Camera\n {error_str}')
+                        self.status = -1
+                        # We dont' run motion energy, or add any files if dlc failed to run
+                        continue
+                    dlc_result = next(self.session_path.joinpath('alf').glob(f'_ibl_{cam}Camera.dlc*.pqt'))
+                    actual_outputs.append(dlc_result)
+
+                    _logger.info(f'Computing motion energy for {cam}Camera')
+                    command2run = f"{self.scripts.joinpath('run_motion.sh')} {str(self.dlcenv)} {file_mp4} {dlc_result}"
+                    _logger.info(command2run)
+                    process = subprocess.Popen(
+                        command2run,
+                        shell=True,
+                        stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE,
+                        executable="/bin/bash",
+                    )
+                    info, error = process.communicate()
+                    info_str = info.decode("utf-8").strip()
+                    _logger.info(info_str)
+                    if process.returncode != 0:
+                        error_str = error.decode("utf-8").strip()
+                        _logger.error(f'Motion energy failed for {cam}Camera \n {error_str}')
+                        self.status = -1
+                        continue
+                    actual_outputs.append(next(self.session_path.joinpath('alf').glob(
+                        f'{cam}Camera.ROIMotionEnergy*.npy')))
+                    actual_outputs.append(next(self.session_path.joinpath('alf').glob(
+                        f'{cam}ROIMotionEnergy.position*.npy')))
+            except BaseException:
+                _logger.error(traceback.format_exc())
+                self.status = -1
+                continue
+        # If status is Incomplete, check that there is at least one output.
+        # # Otherwise make sure it gets set to Empty (outputs = None), and set status to -1 to make sure it doesn't slip
+        if self.status == -3 and len(actual_outputs) == 0:
+            actual_outputs = None
+            self.status = -1
+        return actual_outputs
 
 
 class EphysPostDLC(tasks.Task):
@@ -870,86 +1064,144 @@ class EphysPostDLC(tasks.Task):
                                  ('_ibl_rightCamera.dlc.pqt', 'alf', True),
                                  ('_ibl_rightCamera.times.npy', 'alf', True),
                                  ('_ibl_leftCamera.times.npy', 'alf', True),
-                                 ('_ibl_bodyCamera.times.npy', 'alf', True)],
+                                 ('_ibl_bodyCamera.times.npy', 'alf', True),
+                                 # the following are required for the DLC plot only
+                                 # they are not strictly required, some plots just might be skipped
+                                 # In particular the raw videos don't need to be downloaded as they can be streamed
+                                 ('_iblrig_bodyCamera.raw.mp4', 'raw_video_data', False),
+                                 ('_iblrig_leftCamera.raw.mp4', 'raw_video_data', False),
+                                 ('_iblrig_rightCamera.raw.mp4', 'raw_video_data', False),
+                                 ('rightROIMotionEnergy.position.npy', 'alf', False),
+                                 ('leftROIMotionEnergy.position.npy', 'alf', False),
+                                 ('bodyROIMotionEnergy.position.npy', 'alf', False),
+                                 ('_ibl_trials.choice.npy', 'alf', False),
+                                 ('_ibl_trials.feedbackType.npy', 'alf', False),
+                                 ('_ibl_trials.feedback_times.npy', 'alf', False),
+                                 ('_ibl_trials.stimOn_times.npy', 'alf', False),
+                                 ('_ibl_wheel.position.npy', 'alf', False),
+                                 ('_ibl_wheel.timestamps.npy', 'alf', False),
+                                 ],
+                 # More files are required for all panels of the DLC QC plot to function
                  'output_files': [('_ibl_leftCamera.features.pqt', 'alf', True),
                                   ('_ibl_rightCamera.features.pqt', 'alf', True),
-                                  ('licks.times.npy', 'alf', True)]
+                                  ('licks.times.npy', 'alf', True),
+                                  ('dlc_qc_plot.png', 'snapshot', False)]
                  }
 
-    def _run(self, overwrite=False, run_qc=True):
+    def _run(self, overwrite=False, run_qc=True, plot_qc=True):
+        """
+        Run the EphysPostDLC task. Returns a list of file locations for the output files in signature. The created plot
+        (dlc_qc_plot.png) is not returned, but saved in session_path/snapshots and uploaded to Alyx as a note.
+
+        :param overwrite: bool, whether to recompute existing output files (default is False).
+                          Note that the dlc_qc_plot will be (re-)computed even if overwrite = False
+        :param run_qc: bool, whether to run the DLC QC (default is True)
+        :param plot_qc: book, whether to create the dlc_qc_plot (default is True)
+
+        """
         # Check if output files exist locally
         exist, output_files = self.assert_expected(self.signature['output_files'], silent=True)
         if exist and not overwrite:
-            _logger.warning('EphysPostDLC outputs exist and overwrite=False, skipping.')
-            return output_files
-        if exist and overwrite:
-            _logger.warning('EphysPostDLC outputs exist and overwrite=True, overwriting existing outputs.')
-        # Find all available dlc traces and dlc times
-        dlc_files = list(Path(self.session_path).joinpath('alf').glob('_ibl_*Camera.dlc.*'))
-        for dlc_file in dlc_files:
-            _logger.debug(dlc_file)
-        output_files = []
-        combined_licks = []
+            _logger.warning('EphysPostDLC outputs exist and overwrite=False, skipping computations of outputs.')
+        else:
+            if exist and overwrite:
+                _logger.warning('EphysPostDLC outputs exist and overwrite=True, overwriting existing outputs.')
+            # Find all available dlc files
+            dlc_files = list(Path(self.session_path).joinpath('alf').glob('_ibl_*Camera.dlc.*'))
+            for dlc_file in dlc_files:
+                _logger.debug(dlc_file)
+            output_files = []
+            combined_licks = []
 
-        for dlc_file in dlc_files:
-            # Catch unforeseen exceptions and move on to next cam
-            try:
-                cam = label_from_path(dlc_file)
-                # load dlc trace and camera times
-                dlc = pd.read_parquet(dlc_file)
-                dlc_thresh = likelihood_threshold(dlc, 0.9)
-                # try to load respective camera times
+            for dlc_file in dlc_files:
+                # Catch unforeseen exceptions and move on to next cam
                 try:
+                    cam = label_from_path(dlc_file)
+                    # load dlc trace and camera times
+                    dlc = pd.read_parquet(dlc_file)
+                    dlc_thresh = likelihood_threshold(dlc, 0.9)
+                    # try to load respective camera times
                     dlc_t = np.load(next(Path(self.session_path).joinpath('alf').glob(f'_ibl_{cam}Camera.times.*npy')))
                     times = True
-                except StopIteration:
-                    _logger.error(f'No camera.times found for {cam} camera. '
-                                  f'Computations using camera.times will be skipped')
-                    self.status = -1
-                    times = False
+                    if dlc_t.shape[0] == 0:
+                        _logger.error(f'camera.times empty for {cam} camera. '
+                                      f'Computations using camera.times will be skipped')
+                        self.status = -1
+                        times = False
+                    elif dlc_t.shape[0] < len(dlc_thresh):
+                        _logger.error(f'Camera times shorter than DLC traces for {cam} camera. '
+                                      f'Computations using camera.times will be skipped')
+                        self.status = -1
+                        times = 'short'
+                    # These features are only computed from left and right cam
+                    if cam in ('left', 'right'):
+                        features = pd.DataFrame()
+                        # If camera times are available, get the lick time stamps for combined array
+                        if times is True:
+                            _logger.info(f"Computing lick times for {cam} camera.")
+                            combined_licks.append(get_licks(dlc_thresh, dlc_t))
+                        elif times is False:
+                            _logger.warning(f"Skipping lick times for {cam} camera as no camera.times available")
+                        elif times == 'short':
+                            _logger.warning(f"Skipping lick times for {cam} camera as camera.times are too short")
+                        # Compute pupil diameter, raw and smoothed
+                        _logger.info(f"Computing raw pupil diameter for {cam} camera.")
+                        features['pupilDiameter_raw'] = get_pupil_diameter(dlc_thresh)
+                        try:
+                            _logger.info(f"Computing smooth pupil diameter for {cam} camera.")
+                            features['pupilDiameter_smooth'] = get_smooth_pupil_diameter(features['pupilDiameter_raw'],
+                                                                                         cam)
+                        except BaseException:
+                            _logger.error(f"Computing smooth pupil diameter for {cam} camera failed, saving all NaNs.")
+                            _logger.error(traceback.format_exc())
+                            features['pupilDiameter_smooth'] = np.nan
+                        # Safe to pqt
+                        features_file = Path(self.session_path).joinpath('alf', f'_ibl_{cam}Camera.features.pqt')
+                        features.to_parquet(features_file)
+                        output_files.append(features_file)
 
-                # These features are only computed from left and right cam
-                if cam in ('left', 'right'):
-                    features = pd.DataFrame()
-                    # If camera times are available, get the lick time stamps for combined array
-                    if times:
-                        _logger.info(f"Computing lick times for {cam} camera.")
-                        combined_licks.append(get_licks(dlc_thresh, dlc_t))
+                    # For all cams, compute DLC qc if times available
+                    if times is True or times == 'short' and run_qc:
+                        # Setting download_data to False because at this point the data should be there
+                        qc = DlcQC(self.session_path, side=cam, one=self.one, download_data=False)
+                        qc.run(update=True)
                     else:
-                        _logger.warning(f"Skipping lick times for {cam} camera as no camera.times available.")
-                    # Compute pupil diameter, raw and smoothed
-                    _logger.info(f"Computing raw pupil diameter for {cam} camera.")
-                    features['pupilDiameter_raw'] = get_pupil_diameter(dlc_thresh)
-                    _logger.info(f"Computing smooth pupil diameter for {cam} camera.")
-                    features['pupilDiameter_smooth'] = get_smooth_pupil_diameter(features['pupilDiameter_raw'], cam)
-                    # Safe to pqt
-                    features_file = Path(self.session_path).joinpath('alf', f'_ibl_{cam}Camera.features.pqt')
-                    features.to_parquet(features_file)
-                    output_files.append(features_file)
+                        if times is False:
+                            _logger.warning(f"Skipping QC for {cam} camera as no camera.times available")
+                        if not run_qc:
+                            _logger.warning(f"Skipping QC for {cam} camera as run_qc=False")
 
-                # For all cams, compute DLC qc if times available
-                if times and run_qc:
-                    # Setting download_data to False because at this point the data should be there
-                    qc = DlcQC(self.session_path, side=cam, one=self.one, download_data=False)
-                    qc.run(update=True)
-                else:
-                    if not times:
-                        _logger.warning(f"Skipping QC for {cam} camera as no camera.times available")
-                    if not run_qc:
-                        _logger.warning(f"Skipping QC for {cam} camera as run_qc=False")
+                except BaseException:
+                    _logger.error(traceback.format_exc())
+                    self.status = -1
+                    continue
 
+            # Combined lick times
+            if len(combined_licks) > 0:
+                lick_times_file = Path(self.session_path).joinpath('alf', 'licks.times.npy')
+                np.save(lick_times_file, sorted(np.concatenate(combined_licks)))
+                output_files.append(lick_times_file)
+            else:
+                _logger.warning("No lick times computed for this session.")
+
+        if plot_qc:
+            _logger.info("Creating DLC QC plot")
+            try:
+                session_id = self.one.path2eid(self.session_path)
+                fig_path = self.session_path.joinpath('snapshot', 'dlc_qc_plot.png')
+                if not fig_path.parent.exists():
+                    fig_path.parent.mkdir(parents=True, exist_ok=True)
+                fig = dlc_qc_plot(self.session_path, one=self.one)
+                fig.savefig(fig_path)
+                fig.clf()
+                snp = ReportSnapshot(self.session_path, session_id, one=self.one)
+                snp.outputs = [fig_path]
+                snp.register_images(widths=['orig'],
+                                    function=str(dlc_qc_plot.__module__) + '.' + str(dlc_qc_plot.__name__))
             except BaseException:
+                _logger.error('Could not create and/or upload DLC QC Plot')
                 _logger.error(traceback.format_exc())
                 self.status = -1
-                continue
-
-        # Combined lick times
-        if len(combined_licks) > 0:
-            lick_times_file = Path(self.session_path).joinpath('alf', 'licks.times.npy')
-            np.save(lick_times_file, sorted(np.concatenate(combined_licks)))
-            output_files.append(lick_times_file)
-        else:
-            _logger.warning("No lick times computed for this session.")
 
         return output_files
 
@@ -1025,5 +1277,6 @@ class EphysExtractionPipeline(tasks.Pipeline):
         tasks["EphysCellsQc"] = EphysCellsQc(self.session_path, parents=[tasks["SpikeSorting"]])
         tasks["EphysDLC"] = EphysDLC(self.session_path, parents=[tasks["EphysVideoCompress"]])
         # level 3
-        tasks["EphysPostDLC"] = EphysPostDLC(self.session_path, parents=[tasks["EphysDLC"]])
+        tasks["EphysPostDLC"] = EphysPostDLC(self.session_path, parents=[tasks["EphysDLC"], tasks["EphysTrials"],
+                                                                         tasks["EphysVideoSyncQc"]])
         self.tasks = tasks
