@@ -482,3 +482,37 @@ class TestsSpikeGLX_Meta(unittest.TestCase):
         chns = np.r_[np.arange(30, 101), np.arange(250, 301), 384]
         subset = spikeglx._get_savedChans_subset(chns)
         self.assertEqual(subset, '30:100,250:300,384')
+
+
+class TestsBasicReader(unittest.TestCase):
+    """
+    Tests the basic usage where there is a flat binary and no metadata associated
+    """
+    def test_read_flat_binary_float32(self):
+        # here we expect no scaling to V applied and no sync trace as the format is float32
+        kwargs = dict(ns=60000, nc=384, fs=30000, dtype=np.float32)
+        data = np.random.randn(kwargs['ns'], kwargs['nc']).astype(np.float32)
+        with tempfile.NamedTemporaryFile() as tf:
+            with open(tf.name, mode='w') as fp:
+                data.tofile(fp)
+            sr = spikeglx.Reader(tf.name, **kwargs)
+            assert np.all(sr[:, :] == data)
+            assert sr.nsync == 0
+            assert np.all(sr.sample2volts == 1)
+
+    def test_read_flat_binary_int16(self):
+        # here we expect scaling on all channels but the sync channel
+        np.random.seed(42)
+        kwargs = dict(ns=60000, nc=385, fs=30000, dtype=np.int16)
+        s2v = np.ones(385) * spikeglx.S2V_AP
+        s2v[-1] = 1
+        data = np.random.randn(kwargs['ns'], kwargs['nc']) / s2v
+        data[:, -1] = 1
+        data = data.astype(np.int16)
+        with tempfile.NamedTemporaryFile() as tf:
+            with open(tf.name, mode='w') as fp:
+                data.tofile(fp)
+            sr = spikeglx.Reader(tf.name, **kwargs)
+            assert np.all(np.isclose(sr[:, :-1], data[:, :-1].astype(np.float32) * spikeglx.S2V_AP))
+            assert sr.nsync == 1
+            assert np.all(sr.sample2volts == s2v)
